
#ifndef _RANSAC_CPP_
#define _RANSAC_CPP_
//if no boost available, then comment the next line out, builtin Rand() will kick in.
//#include <boost/version.hpp>
#if BOOST_VERSION >= 103301
#define _USE_BOOST_PRNG_
#endif
#define NOMINMAX

#include "ransac.h"
#include "homography2d.h"
#include "configs.h"
#include <iostream>
#include <cmath>
#include <vector>
#include <algorithm>

#define max(a,b) ((a) > (b)) ? (a) : (b)
#define min(a,b) ((a) < (b)) ? (a) : (b)
//#define _VERBOSE_FLAG_

#ifdef _USE_BOOST_PRNG_ 
#include <ctime>            // std::time
#include <boost/random.hpp>
// Sun CC doesn't handle boost::iterator_adaptor yet
#if !defined(__SUNPRO_CC) || (__SUNPRO_CC > 0x530)
#include <boost/generator_iterator.hpp>
#endif
#else
//use native code: Mersenne Twister
#endif

#ifndef _USE_BOOST_PRNG_
/* 
A C-program for MT19937, with initialization improved 2002/1/26.
Coded by Takuji Nishimura and Makoto Matsumoto.

Before using, initialize the state by using init_genrand(seed)  
or init_by_array(init_key, key_length).

Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
All rights reserved.                          

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

3. The names of its contributors may not be used to endorse or promote 
products derived from this software without specific prior written 
permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEqUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


Any feedback is very welcome.
http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)

Reference: M. Matsumoto and T. Nishimura, "Mersenne Twister: 
A 623-Dimensionally Equidistributed Uniform Pseudo-Random Number Generator",
ACM Transactions on Modeling and Computer Simulation, Vol. 8, No. 1,
January 1998, pp 3--30. 
*/

// C++ wrapped up by Changjiang Yang, more important: make it thread-safe.
namespace MT {
/* Period parameters */  
const int N = 624;
const int M = 397;
const unsigned long MATRIX_A = 0x9908b0dfUL;   /* constant vector a */
const unsigned long UPPER_MASK = 0x80000000UL; /* most significant w-r bits */
const unsigned long LOWER_MASK = 0x7fffffffUL; /* least significant r bits */

class MersenneTwist {
    unsigned long mt[N]; /* the array for the state vector  */
    int mti; /* mti==N+1 means mt[N] is not initialized */
public:

    MersenneTwist() : mti(N+1) {}

    /* initializes mt[N] with a seed */
    //   Note:  Initializing TWISTER to the scalar integer state 0 actually
    //   corresponds to the C call init_genrand(5489).
    void init_genrand(unsigned long s)
    {
        mt[0]= s & 0xffffffffUL;
        for (mti=1; mti<N; mti++) {
            mt[mti] = 
                (1812433253UL * (mt[mti-1] ^ (mt[mti-1] >> 30)) + mti); 
            /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
            /* In the previous versions, MSBs of the seed affect   */
            /* only MSBs of the array mt[].                        */
            /* 2002/01/09 modified by Makoto Matsumoto             */
            mt[mti] &= 0xffffffffUL;
            /* for >32 bit machines */
        }
    }

    /* initialize by an array with array-length */
    /* init_key is the array for initializing keys */
    /* key_length is its length */
    /* slight change for C++, 2004/2/26 */
    void init_by_array(unsigned long init_key[], int key_length)
    {
        int i, j, k;
        init_genrand(19650218UL);
        i=1; j=0;
        k = (N>key_length ? N : key_length);
        for (; k; k--) {
            mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1664525UL))
                + init_key[j] + j; /* non linear */
            mt[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
            i++; j++;
            if (i>=N) { mt[0] = mt[N-1]; i=1; }
            if (j>=key_length) j=0;
        }
        for (k=N-1; k; k--) {
            mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1566083941UL))
                - i; /* non linear */
            mt[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
            i++;
            if (i>=N) { mt[0] = mt[N-1]; i=1; }
        }

        mt[0] = 0x80000000UL; /* MSB is 1; assuring non-zero initial array */ 
    }

    /* generates a random number on [0,0xffffffff]-interval */
    unsigned long genrand_int32(void)
    {
        unsigned long y;
        static unsigned long mag01[2]={0x0UL, MATRIX_A};
        /* mag01[x] = x * MATRIX_A  for x=0,1 */

        if (mti >= N) { /* generate N words at one time */
            int kk;

            if (mti == N+1)   /* if init_genrand() has not been called, */
                init_genrand(5489UL); /* a default initial seed is used */

            for (kk=0;kk<N-M;kk++) {
                y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
                mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1UL];
            }
            for (;kk<N-1;kk++) {
                y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
                mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
            }
            y = (mt[N-1]&UPPER_MASK)|(mt[0]&LOWER_MASK);
            mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1UL];

            mti = 0;
        }

        y = mt[mti++];

        /* Tempering */
        y ^= (y >> 11);
        y ^= (y << 7) & 0x9d2c5680UL;
        y ^= (y << 15) & 0xefc60000UL;
        y ^= (y >> 18);

        return y;
    }

    /* generates a random number on [0,0x7fffffff]-interval */
    long genrand_int31(void)
    {
        return (long)(genrand_int32()>>1);
    }

    /* generates a random number on [0,1]-real-interval */
    double genrand_real1(void)
    {
        return genrand_int32()*(1.0/4294967295.0); 
        /* divided by 2^32-1 */ 
    }

    /* generates a random number on [0,1)-real-interval */
    double genrand_real2(void)
    {
        return genrand_int32()*(1.0/4294967296.0); 
        /* divided by 2^32 */
    }

    /* generates a random number on (0,1)-real-interval */
    double genrand_real3(void)
    {
        return (((double)genrand_int32()) + 0.5)*(1.0/4294967296.0); 
        /* divided by 2^32 */
    }

    /* generates a random number on [0,1) with 53-bit resolution*/
    double genrand_res53(void) 
    { 
        unsigned long a=genrand_int32()>>5, b=genrand_int32()>>6; 
        return(a*67108864.0+b)*(1.0/9007199254740992.0); 
    }
};
}
#endif

/*
% RANSAC - Robustly fits a model to data with the RANSAC algorithm
%
% Usage:
%
% int ransac(x, rows, npts, s, t, M, inliers)
%
% Inputs:
%     x         - Data sets to which we are seeking to fit a model M
%                 It is assumed that x is of size [d x Npts]
%                 where d is the dimensionality of the data and Npts is
%                 the number of data points.
%
%     s         - The minimum number of samples from x required by
%                 fittingfn to fit a model.
%
%     t         - The distance threshold between a data point and the model
%                 used to decide whether the point is an inlier or not.
%
% Outputs:
%     M         - The model having the greatest number of inliers.
%     inliers   - An array of indices of the elements of x that were
%                 the inliers for the best model.
% References:
%    M.A. Fishler and  R.C. Boles. "Random sample concensus: A paradigm
%    for model fitting with applications to image analysis and automated
%    cartography". Comm. Assoc. Comp, Mach., Vol 24, No 6, pp 381-395, 1981
%
%    Richard Hartley and Andrew Zisserman. "Multiple View Geometry in
%    Computer Vision". pp 117-131. Cambridge University Press, 2001
*/
//template<typename _DegenerateFunction, typename  _FittingFunction, typename _DistanceFunction>
int ransac(ProjectiveTransformation::WarpModel wm, DegeneracyCheck::Level dc, const double *x, const int rows, const int npts, const int s, const double t, 
           double M[9], std::vector<int>& inliers, 
		   bool (*DegenFnct)(ProjectiveTransformation::WarpModel, DegeneracyCheck::Level, const double *, const int),
		   int (*FittingFnct)(ProjectiveTransformation::WarpModel, double *, const int, const int, double []),
		   int (*DistFnct)(const double [], const double *, const int, const int, const double, int*),
           const int maxDegenTrials,
           const int minRansacTrials,
           const int maxRansacTrials)
{
    int trialcount = 0;
    int bestscore = 0;
    int ninliers = 0;

    int N = 1;

    double bestM[9];
    //int *bestinliers = new int[npts];
    std::vector<int> bestinliers(npts);
    //double *xrnd4 = new double[s*rows];
    std::vector<double> xrnd4(s*rows);

#ifdef _USE_BOOST_PRNG_ 
    boost::mt19937 rng;     //pseudo-random number generators (Mersenne Twister)
    //rng.seed(static_cast<unsigned int>(std::time(0)));
    //srand(static_cast<unsigned int>(std::time(0)));
    rng.seed(0);
    boost::uniform_int< > uni(0,npts-1); // distribution that maps to 0..npts-1
    // see random number distributions
    boost::variate_generator<boost::mt19937&, boost::uniform_int< > > 
        die(rng, uni);             // glues randomness with mapping
#else
    MT::MersenneTwist mtrnd;
    mtrnd.init_genrand(5489UL); //initialize the Mersenne Twister.
#endif
    while (trialcount < minRansacTrials || trialcount < N)
    {
        /*
        % Select at random s datapoints to form a trial model, M.
        % In selecting these points we have to check that they are not in
        % a degenerate configuration.
        */
        bool degenerate = true;
        int count = 1;
        while (degenerate)
        {
            //Generate s random indicies in the range 1..npts
            std::vector<double>::iterator pxrnd4 = xrnd4.begin();
            for (int i = 0; i < s; i++)
            {
                //int irnd = rand() % npts;
#ifdef _USE_BOOST_PRNG_
                int irnd = die();               // simulate rolling a die
#else
                int irnd = static_cast<int>(floor(mtrnd.genrand_res53()*npts));
                //irnd = std::min(irnd, npts-1);
#endif
                pxrnd4 = std::copy(x+irnd*rows, x+irnd*rows+rows, pxrnd4);
            }

            degenerate = DegenFnct(wm, dc, &xrnd4[0], rows);

            if (!degenerate)
            {
                int info = FittingFnct(wm, &xrnd4[0], rows, s, M);
                if (info != 0)
                {
                    degenerate = true;
                }
            }// end of if (!is_degenerate)

            // Safeguard against being stuck in this loop forever
            ++count;
            if (count > maxDegenTrials)
            {
#ifdef _VERBOSE_FLAG_
                std::cerr << "Unable to select a nondegenerate data set!"<<std::endl;
#endif
                degenerate = true;
                //				delete []bestinliers;
                return RansacErrorCode::RANSAC_NO_NONDEGENERATE_DATA;
            }
        }// end of == while degenerate ==

        /*
        Once we are out here we should have some kind of model...        
        Evaluate distances between points and model returning the indices
        of elements in x that are inliers.  M is an array of possible models 'distfn' 
        will return the model that has the most inliers.
        */
        ninliers = DistFnct(M, x, rows, npts, t, &inliers[0]);

        if (ninliers > bestscore)
        {
            bestscore = ninliers;

            //std::copy(inliers, inliers+ninliers, bestinliers);
            bestinliers.swap(inliers);
            std::copy(M, M+9, bestM);

            float fracinliers =  static_cast<float>(ninliers)/static_cast<float>(npts);
            float pNoOutliers = 1.0f -  pow(fracinliers, s);

            pNoOutliers = max(pNoOutliers, RansacSetting::flt_eps_2);	// Avoid division by -Inf
            pNoOutliers = min(1.0f-RansacSetting::flt_eps_2, pNoOutliers);// Avoid division by 0
            const float log1minusp = log(1.0f - RansacSetting::pconfidence);
            N = static_cast<int> (log1minusp / log(pNoOutliers) + 0.5f);
        }// end of if (ninliers > bestscore)

        ++trialcount;

        if (trialcount > maxRansacTrials)
        {
#ifdef _VERBOSE_FLAG_
            std::cerr << "RANSAC reached the maximum number of " << maxRansacTrials << " trials"<<std::endl;
#endif
            break;
        }
    }

    if (bestscore > 0) // We got a solution
    {
        std::copy(bestM, bestM+9, M);
        //std::copy(bestinliers, bestinliers+bestscore, inliers);
        bestinliers.swap(inliers);
    }
    else
    {
#ifdef _VERBOSE_FLAG_
        std::cerr << "RANSAC was unable to find a useful solution." << std::endl;
#endif
        bestscore = RansacErrorCode::RANSAC_NO_SOLUTION;
    }

    //delete []bestinliers;
    return bestscore;
}

#endif