
#ifndef _RANSACFITHOMOGRAPHY_H_
#define _RANSACFITHOMOGRAPHY_H_

#include "ransacfithomography_defs.h"
#include "configs.h"

/*
* RANSACFITHOMOGRAPHY - fits 2D homography using RANSAC
*
* Usage:   int ransacfithomography(x1, x2, rows, npts, t, H, inliers)
*
* Inputs:
*          x1  - rows x npts set of homogeneous points.  If the data is
*                2xN it is assumed the homogeneous scale factor is 1.
*          x2  - rows x npts set of homogeneous points such that x1<->x2.
*				 x1 and x2 are 1D double arrays.
*		   rows- Dimensionality of input data. 2 if nonhomogeneous data,
*				 3 if homogeneous data.
*		   npts- Number of points.
*          t   - The distance threshold between data point and the model
*                used to decide whether a point is an inlier or not. 
*                Note that point coordinates are normalized to that their
*                mean distance from the origin is sqrt(2).  The value of
*                t should be set relative to this, say in the range 
*                0.001 - 0.01  
*
* Outputs:
*          H       - The 3x3 homography such that x2 = H*x1. (Column major)
*          inliers - An array of indices of the elements of x1, x2 that were
*                    the inliers for the best model.
*		   Function return - The number of the inliers and ransac information.
*				= -1: Must have at least 4 points to fit homography;
*				= -2: x1 and x2 must have 2 or 3 rows;
*				= -3: No least-square solution for the inliers;
*				= -4: No nondegenerate data set;
*				= -5: Ransac has no useful solution
* Reference:
*		Adapted from Peter Kovesi's matlab function code:
*		http://www.csse.uwa.edu.au/~pk/research/matlabfns/
*/

/* Usage for template arguments:
*  ProjectiveTransformation::WarpModel wm: 
*        Choose the projective transformation model (Translation,Similarity, Affine, Homography)
*  DegeneracyCheck::Level dc:
*        Check for Degeneracy, the higher level, the more strict check at the price of more computations. 
*  int minSizeSample:
*        Usually 4 for homgography, 3 for affine, 2 for similarity, 1 for translation. But you can
*        increase it to remove the warping ambiguities at the price of larger number of trials.
*  Note: the argument "minSizeSample" put in template is due to the fact that it is a RANSAC
*        model parameter. I want to let the user to know that all the template arguments are
*        for RANSAC model, all function arguments are for user input/output data. Purely for
*        interface unification.
*/
//RANSACFITHOMOGRAPHY_EXPORT 
/*
template <ProjectiveTransformation::WarpModel wm, DegeneracyCheck::Level dc, const int minSizeSample>
int ransacfithomography(const double *x1, const double *x2, const int rows, const int npts, 
                        const double t, double H[9], int *inliers);
*/
int ransacfithomography(ProjectiveTransformation::WarpModel wm, DegeneracyCheck::Level dc, const int minSizeSample, 
						const double *x1, const double *x2, const int rows, const int npts, 
                        const double t, double H[9], int *inliers);

#endif
