
#ifndef _OBJECTMODELLER_H
#define _OBJECTMODELLER_H

#include "TheTracker.h"
#include <list>
#include <deque>

struct RegionAttributes {
public:
	int m_label;
	// Spatial Model
	int m_area;
	Point m_centroid;
	double m_region_mean[2];
	double m_region_cov[2][2];
	double m_inv_region_cov[2][2];
	double vel_x, vel_y;
	
	//occlusion case (predicted centroid n velocity)
	Point m_occ_centroid;
	double occ_vel_x,occ_vel_y;
	
	Rect m_tracking_context;
	Rect m_bounding_box;

	// Appearance Model
	double m_feature_mean[3];
	double m_feature_cov[3][3];
	double m_inv_feature_cov[3][3];
	bool m_is_separable;
	list<Point> m_pixel_list;
	list<Point> m_nonOccluded_pixel_list;

	// Region Occlusion
	bool m_is_occluded;
	bool m_isValid;
	
	RegionAttributes()
	{
		m_centroid.x = 0;m_centroid.y=0;
		m_isValid = true;
	}

	void SetFeatureMean(vector<double>& feature_mean) {
		for(int i=0; i<feature_mean.size(); i++)
			m_feature_mean[i] = feature_mean[i];
	}

	void SetFeatureMean(double r_mean, double g_mean, double b_mean) {
		m_feature_mean[0] = r_mean;
		m_feature_mean[1] = g_mean;
		m_feature_mean[2] = b_mean;
	}

	void SetRegionMean(double x_mean, double y_mean) {
		m_region_mean[0] = x_mean;
		m_region_mean[1] = y_mean;
	}

	void SetFeatureCovariance(double cov[3][3]) {
		int i, j;
		for(i=0; i<3; i++)
			for(j=0; j<3; j++)
				m_feature_cov[i][j] = cov[i][j];
	}

	void SetRegionCovariance(double cov[2][2]) {
		m_region_cov[0][0] = cov[0][0];
		m_region_cov[0][1] = cov[0][1];
		m_region_cov[1][0] = cov[1][0];
		m_region_cov[1][1] = cov[1][1];
	}

	RegionAttributes& operator=(const RegionAttributes& src) {
		if(this == &src)
			return *this;
		int i, j;
		m_label = src.m_label;
		m_bounding_box = src.m_bounding_box;
		m_centroid = src.m_centroid;
		for(i=0; i<3; i++) {
			for(j=0; j<3; j++) {
				m_feature_cov[i][j] = src.m_feature_cov[i][j];
				m_inv_feature_cov[i][j] = src.m_inv_feature_cov[i][j];
			}
		}
		for(i=0; i<3; i++)
			m_feature_mean[i] = src.m_feature_mean[i];
		
		for(i=0; i<2; i++) {
			for(j=0; j<2; j++) {
				m_region_cov[i][j] = src.m_region_cov[i][j];
				m_inv_region_cov[i][j] = src.m_inv_region_cov[i][j];
			}
		}
		m_region_mean[0] = src.m_region_mean[0];
		m_region_mean[1] = src.m_region_mean[1];
		m_is_separable = src.m_is_separable;
		m_tracking_context = src.m_tracking_context;
		m_pixel_list.clear();
		for(list<Point>::const_iterator list_iter = src.m_pixel_list.begin(); list_iter != src.m_pixel_list.end(); list_iter++)
			m_pixel_list.push_back(*list_iter);
		return *this;
	}
};

ostream& operator <<(ostream& out, const RegionAttributes& att);


struct Shape {
	vector<Point> contour_points;
	Point centroid;
	int size;
	boolean IsObjectOccluded;
};

/*
   Some of the functions in the ObjectModel have 2 variants. One variant directly updates the member variables. The other 
   is written as a generic function. Hence the generic functions does not update its member variable and the caller can 
   pass the variable to be updated as an argument.
*/
class ObjectModel {
public:

	int m_width, m_height;
	int m_tc_threshold;
	double m_update_weight;		// lambda
	double m_cost_threshold;	// If the cost(1 / Mah-dist) between 2 blobs is greater than this threshold, they r considered as dissimilar

	// Foreground Color and Spatial Model
	vector<RegionAttributes> m_fg_rgn_att;
	vector<RegionAttributes> m_fg_initial_model;
	deque< vector< RegionAttributes > > m_fg_rgn_history;   // allows random access with constant time for insertion n removal of elements at the end
	int m_no_past_frames;	// number of past frames to be considered for updating foreground appearance model

	// Background Color and Spatial Model
	vector<RegionAttributes> m_bkg_rgn_att;
	vector<RegionAttributes> m_bg_initial_model;
	deque< vector< RegionAttributes > > m_bg_rgn_history;

	// Object Attributes - only spatial model
	RegionAttributes m_object_att;

	// Shape
	vector<Point> m_contour_points;
	vector<Point> m_prev_contour_points;
	vector<Shape> m_contour_history;

	// Edge
	ImgBinary m_edge_image;

	ObjectModel(int width, int height);

	// Overall Object model (handles only spatial information and not appearance information)
	void CalcObjectAttributes(const ImgInt& labels, RegionAttributes& rgn_att);
	void CalcObjectAttributes(const ImgInt& labels);
	bool UpdateObjectAttributes(const ImgBinary& mask, RegionAttributes& rgn_att, int& dx, int& dy);
	bool UpdateObjectAttributes(const ImgBinary& mask, int& dx, int& dy);

	// Update Spatial model if the region is not occluded. If occluded, assign the spatial model of the object to the region
	void UpdateSpatialModel(RegionAttributes& object_att, vector<RegionAttributes>& rgn_att, const ImgBinary& mask, int dx, int dy, bool check_consistency);
	void CopySpatialModel(RegionAttributes& src, RegionAttributes& dest);

	// Compute the spatial model based on the given pixel list
	void ComputeSpatialModel(const list<Point>& pixel_list, RegionAttributes& rgn_att);
	void ComputeSpatialModel(const list<Point>& pixel_list, RegionAttributes& rgn_att, int x_mean, int y_mean, int area);

	// Compute the Appearance model based on the given pixel list 
	void ComputeAppearanceModel(const ImgBgr& input_image, const list<Point>& pixel_list, RegionAttributes& rgn_att);

	// Update Shape
	void UpdateObjectHistory(int frame_no, const vector<Point>& contour_pts, const RegionAttributes& object_att, bool isObjectOccluded, vector<Shape> *contour_history);
	void UpdateObjectHistory(int frame_no, bool isObjectOccluded);

	// Generates the spatial and appearance model of all regions and the spatial model of the object. Called during initialization.
	void GenerateObjectModel(const ImgBgr& input_image);

	// Update the edges.
	void UpdateEdge(const ImgBgr& input_image);

	// Update the appearance model using a function of past and present statistics
	void UpdateAppearanceModel(const ImgBgr& current_frame, vector<RegionAttributes>& rgn_att, vector<RegionAttributes>& initial_model, deque< vector< RegionAttributes > > *rgn_history);

	// Update the foreground model - spatial and appearance using the pixel list generated by max prob
	void UpdateFgSpatialModel(const ImgBinary& mask, int dx, int dy, bool check_consistency);
	void UpdateFgAppearanceModel(const ImgBgr& input_image);

	// Update the background model - spatial and appearance using the pixel list generated by max prob
	void UpdateBgSpatialModel(const ImgBinary& mask);
	void UpdateBgAppearanceModel(const ImgBgr& current_frame);

	// Include new modes, if any, in the appearance model and create corresponding spatial models for them. This helps in handling self-occlusion
	void UpdateNewModes(const ImgBgr& current_frame, const ImgBgr& previous_frame, const ImgFloat& strength_image, const ImgBinary& object_mask);
	void AddNewMode(const Rect& bb, const ImgBinary& mask, const ImgBgr& current_frame, vector<RegionAttributes>& model, vector<RegionAttributes>& initial_model);
};

#endif