
#include "RegionSegmentation.h"
#include "utils\Utils.h"
#include <iostream>
#include <stack>

ImgInt* RegionSegmentation::m_labels = NULL;
list<RegionInfo>* RegionSegmentation::m_regions = NULL;

bool _comp(Density& density1, Density& density2)
{
	return (density1.m_density > density2.m_density);
}

RegionSegmentation::RegionSegmentation()
{
	m_density_window = 9;
	m_num_features = 3;
	m_current_label_no = 1;

	// Parameters that define the quality of the segmentation output.
	m_sigma_th = 2.0f;
	m_area_th = 0.09f;

	m_fg_total_ratio = 50;
	m_fg_mask_ratio = 25;
}

void RegionSegmentation::FreeMemory()
{
	if(m_labels != NULL) {
		delete m_labels;
		m_labels = NULL;
	}
	if(m_regions != NULL) {
		delete m_regions;
		m_regions = NULL;
	}
}

void RegionSegmentation::DoSegmentation(const ImgBgr& image, bool classify_edges, const ImgBinary& input_mask, double sigma, vector<RegionAttributes> *fg_rgns, vector<RegionAttributes> *bg_rgns)
{
	int width = image.Width();
	int height = image.Height();
	m_sigma_th = sigma;

	if(m_labels == NULL) {
		m_labels = new ImgInt();
		m_labels->Reset(width, height);
		if(m_regions == NULL)
			m_regions = new list<RegionInfo>;
		_SegmentImage(image, input_mask);
	}

	if(m_regions == NULL) {
		m_regions = new list<RegionInfo>;
		// Generate region info from label image
	}

	_RemoveSmallRegions(width*height, m_regions, *m_labels);

	if(classify_edges)
		_ClassifyEdgePixels(image, width, height);

	// Segment regions into fg and bg regions based on the given mask
	_SegmentRegion(input_mask, fg_rgns, bg_rgns);
	//=================================================================
	ImgInt tmp_labels(width, height);
	Set(&tmp_labels, 0);
	for(int i=0; i<fg_rgns->size(); i++) {
		for(list<Point>::iterator pixel_iter = (*fg_rgns)[i].m_pixel_list.begin(); pixel_iter != (*fg_rgns)[i].m_pixel_list.end(); pixel_iter++) {
			tmp_labels((*pixel_iter).x, (*pixel_iter).y) = i+1;
		}
	}
	ImgBgr pseudo_color;
	Utils::RandomColor(tmp_labels, &pseudo_color);
	Figure fig("**Foreground Segmentation");  fig.SetWindowPosition(0, height+70);
	fig.Draw(pseudo_color);
	Set(&tmp_labels, 0);
	for(i=0; i<bg_rgns->size(); i++) {
		for(list<Point>::iterator pixel_iter = (*bg_rgns)[i].m_pixel_list.begin(); pixel_iter != (*bg_rgns)[i].m_pixel_list.end(); pixel_iter++)
			tmp_labels((*pixel_iter).x, (*pixel_iter).y) = i+1;
	}
	Utils::RandomColor(tmp_labels, &pseudo_color);
	Figure bg_wnd("**Background Segmentation");  bg_wnd.PlaceToTheRightOf(fig);
	bg_wnd.Draw(pseudo_color);
	
	//==================================================================
}

/*
	The function segments the image using the following procedure:
		1) Compute density map	(_ComputeDensityMap())
		2) Repeat until all pixels are classified
			a) Pick seed point (unclassified pixel with max density) from density map
			b) Grow region based on color similarity	(_GrowRegion())
			c) Generate a running statistics while growing the region.

	Input:	image - input image
			input_mask - used to calculate the foreground area.
	Output: updates m_regions - statistics (pixel list, area) of all the regions.

	m_label - +ve -> Pixel classified
			- 0   -> Pixle unclassified yet
			- -1  -> Edge pixel. Do not consider this for segmentation.
*/
void RegionSegmentation::_SegmentImage(const ImgBgr& image, const ImgBinary& input_mask)
{
	int pos, x, y;
	int size;
	vector<Density> density_list;
	int width = image.Width();
	int height = image.Height();
	Set(m_labels, 0);
	//Figure fig;

	// Canny edge
	ImgGray image_gray;
	ImgBinary edge_mask;
	Convert(image, &image_gray);
	Canny(image_gray, &edge_mask, 2.0f);
	Dilate3x3(edge_mask, &edge_mask); 

	// Set all the pixels in the label image to -1 where gradient is high
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++) {
			if(edge_mask(x,y)) (*m_labels)(x,y) = -1;
		}
	}

	// Compute the densities and sort in descending order based on densities. High densities represent similar looking pixels in the neighborhood.
	ImgBinary mask(width, height);
	Set(&mask, 1);
	_ComputeDensityMap(image, mask, &density_list);
	sort(density_list.begin(), density_list.end(), _comp);

	pos = 0;
	size = density_list.size();
	while(pos < size) {
		if((*m_labels)(density_list[pos].m_pt.x, density_list[pos].m_pt.y) != 0) {	// if the pixel is already classified or edge, continue
			pos++;
			continue;
		}
		RegionInfo rgn_info;
		_GrowRegion(image, density_list[pos].m_pt, input_mask, rgn_info, *m_labels);
		m_regions->push_back(rgn_info);
		//=================================================================
		/*
		ImgBgr pseudo_region;
		Utils::RandomColor(*m_labels, &pseudo_region);
		fig.Draw(pseudo_region);
		*/
		//=================================================================
		m_current_label_no++;
		pos++;
	}
}

/*
	This function computes a density map using a pre-defined kernel size. (default = 5)
	density(x,y) = no of points in the kernel / volume encompassed by the points in the feature space
		where no of points in kernel = kernel size * kernel size,
			  volume = (max_r - min_r) * (max_g - min_g) * (max_b - min_b)
	Only the pixels set in the mask are considered for computing the density map. This helps in segmenting only 
	specific regions of the image.
*/
void RegionSegmentation::_ComputeDensityMap(const ImgBgr& image, const ImgBinary& mask, vector<Density> *density_list)
{
	int x, y, xx, yy;
	int min_r, min_g, min_b;
	int max_r, max_g, max_b;

	int width = image.Width();
	int height = image.Height();

	int hdw = m_density_window/2;

	for(y=hdw; y<height-hdw; y++) {
		for(x=hdw; x<width-hdw; x++) {
			min_r = 256; min_g = 256; min_b = 256;
			max_r = -1; max_g = -1; max_b = -1;

			if(!mask(x,y))
				continue;

			// Find the min and max in the kernel to compute the volume
			for(yy=-hdw; yy<=hdw; yy++) {
				for(xx=-hdw; xx<=hdw; xx++) {
					int r = image(x+xx, y+yy).r; int g = image(x+xx, y+yy).g; int b = image(x+xx, y+yy).b;
					if(r < min_r) min_r = r;
					if(r > max_r) max_r = r;
					if(g < min_g) min_g = g;
					if(g > max_g) max_g = g;
					if(b < min_b) min_b = b;
					if(b > max_b) max_b = b;
				}
			}

			// Compute the density
			int diff_r = max_r - min_r;
			if(diff_r == 0) diff_r = 1;
			int diff_g = max_g - min_g;
			if(diff_g == 0) diff_g = 1;
			int diff_b = max_b - min_b;
			if(diff_b == 0) diff_b = 1;
			float volume = diff_r * diff_g * diff_b;
			float density = (float)(m_density_window * m_density_window)/volume;

			// Insert into the density list
			density_list->push_back(Density(Point(x,y), density));
		}
	}
}	

/*
	This function grows a region based on the similarity of neighboring pixels. Once the region reaches a pre-defined size 
	(say 10), the function maintains a running statistics to decide whether a neighboring pixel is 'similar' to the region
	being grown. The function outputs 4 values: 
		(i)   Total area of the region
		(ii)  Foreground area of the region (input_mask is used to calculate this)
		(iii) Pixel list that belongs to the region. (rgn_info passed as parameter)
		(iv)  Label image updated with the current label number.	(labels passed as parameter)
*/
void RegionSegmentation::_GrowRegion(const ImgBgr& image, const Point& seed_pt, const ImgBinary& input_mask, RegionInfo& rgn_info, ImgInt& labels)
{
	int image_width = image.Width();
	int image_height = image.Height();
	stack<Point> node;
	Point pt;
	int i, x, y;

	// initialize the region info structure
	rgn_info.m_pixel_list.clear();
	rgn_info.m_fg_area = 0;
	rgn_info.m_total_area = 0;

	// initialize the running sum and sum squares vectors.
	int size = 0;
	vector<int> sum;
	vector<int> sum_squares;
	for(i=0; i<m_num_features; i++) {
		sum.push_back(0);
		sum_squares.push_back(0);
	}

	// out of bounds condition
	if(seed_pt.x < 0 || seed_pt.x >= image_width || seed_pt.y < 0 || seed_pt.y >= image_height)
	{
		printf("Seed point is out of bounds.\n");
		return;
	}

	// push the seed point into the stack and initialize the running statistics (sum n std dev)
	ImgBgr::Pixel check_color = image(seed_pt.x, seed_pt.y);
	node.push(seed_pt);
	labels(seed_pt.x, seed_pt.y) = m_current_label_no;
	rgn_info.m_pixel_list.push_front(Point(seed_pt.x, seed_pt.y));
	_UpdateRunningSum(check_color, sum, sum_squares, size);

	while(!node.empty()) {
		// pop the node
		pt = node.top();
		node.pop();
		x = pt.x; y = pt.y;

		// calculate area
		rgn_info.m_total_area++;
		if(input_mask(x,y)) rgn_info.m_fg_area++;

		/*
		  The third condition in each if serves 2 purposes: 
		  1) It does not consider segmented pixels. It tries to do the floodfill only in the unmarked areas.
		  2) It does not consider edge pixels
		*/
		//x-1, y-1
		if(x-1 >= 0 && y-1 >= 0 && _IsSame(image(x-1, y-1), check_color, sum, sum_squares, size) && labels(x-1, y-1) == 0)
		{
			node.push(Point(x-1, y-1));
			_UpdateRunningSum(image(x-1, y-1), sum, sum_squares, size);
			labels(x-1, y-1) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x-1,y-1));
		}
		// x, y-1
		if(y-1 >= 0 && _IsSame(image(x, y-1), check_color, sum, sum_squares, size) && labels(x, y-1) == 0)
		{
			node.push(Point(x, y-1));
			_UpdateRunningSum(image(x, y-1), sum, sum_squares, size);
			labels(x, y-1) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x,y-1));
		}
		// x+1, y-1
		if(y-1 >= 0 && x+1 < image_width && _IsSame(image(x+1,y-1), check_color, sum, sum_squares, size) && labels(x+1, y-1) == 0)
		{
			node.push(Point(x+1,y-1));
			_UpdateRunningSum(image(x+1, y-1), sum, sum_squares, size);
			labels(x+1,y-1) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x+1, y-1));
		}
		// x-1, y
		if(x-1 >= 0 && _IsSame(image(x-1, y), check_color, sum, sum_squares, size) && labels(x-1, y) == 0)
		{
			node.push(Point(x-1, y));
			_UpdateRunningSum(image(x-1, y), sum, sum_squares, size);
			labels(x-1, y) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x-1, y));
		}
		// x+1,y
		if(x+1 < image_width && _IsSame(image(x+1, y), check_color, sum, sum_squares, size) && labels(x+1, y) == 0)
		{
			node.push(Point(x+1, y));
			_UpdateRunningSum(image(x+1, y), sum, sum_squares, size);
			labels(x+1, y) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x+1, y));
		}
		// x-1, y+1
		if(x-1 >= 0 && y+1 < image_height && _IsSame(image(x-1,y+1), check_color, sum, sum_squares, size) && labels(x-1, y+1) == 0)
		{
			node.push(Point(x-1, y+1));
			_UpdateRunningSum(image(x-1, y+1), sum, sum_squares, size);
			labels(x-1, y+1) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x-1, y+1));
		}	
		// x, y+1
		if(y+1 < image_height && _IsSame(image(x, y+1), check_color, sum, sum_squares, size) && labels(x, y+1) == 0)
		{
			node.push(Point(x, y+1));
			_UpdateRunningSum(image(x, y+1), sum, sum_squares, size);
			labels(x, y+1) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x, y+1));
		}
		// x+1, y+1
		if(x+1 < image_width && y+1 < image_height && _IsSame(image(x+1,y+1), check_color, sum, sum_squares, size) && labels(x+1, y+1) == 0)
		{
			node.push(Point(x+1,y+1));
			_UpdateRunningSum(image(x+1, y+1), sum, sum_squares, size);
			labels(x+1,y+1) = m_current_label_no;
			rgn_info.m_pixel_list.push_front(Point(x+1, y+1));
		}

	} // end of while	
}


/*
	This function segments the regions into either background or foreground regions. It might also split a 
	region into background and foreground. The decision is made using total area, foreground area and mask area.

	if fg_area = 0		=>		Add it to bg_rgns
	if fg_area / area  < fg_total_ratio (50%)
		a) if fg_area / mask_area < fg_mask_ratio (25%)  ->  remove the fg part of the region from the pixel list and add it to bg_rgns
		b) else  =>  split the region into two - a fg_rgn and a bg_rgn and add them
	else
		=> Add it to fg_rgns
		
*/
void RegionSegmentation::_SegmentRegion(const ImgBinary& input_mask, vector<RegionAttributes> *fg_rgns, vector<RegionAttributes> *bg_rgns)
{
	int x, y;
	int height = input_mask.Height();
	int width = input_mask.Width();
	list<RegionInfo>::iterator rgn_iter;
	list<Point>::iterator pixel_iter;

	// compute the mask area
	int mask_area = 0;
	for(y=0; y<height; y++) {
		for(x=0; x<width; x++) {
			if(input_mask(x,y)) mask_area++;
		}
	}
	
	// for each region
	for(rgn_iter = m_regions->begin(); rgn_iter != m_regions->end(); rgn_iter++) {
		RegionInfo& rgn_info = *rgn_iter;

		// If there is no foreground, add it to the bg_rgns
		if(rgn_info.m_fg_area == 0) {
			RegionAttributes rgn_att;
			rgn_att.m_area = rgn_info.m_total_area;
			rgn_att.m_pixel_list = rgn_info.m_pixel_list;
			bg_rgns->push_back(rgn_att);
			continue;
		}
		
		// If 50% of the region is inside the foreground...
		if(((double)rgn_info.m_fg_area / (double)rgn_info.m_total_area) * 100.0f < m_fg_total_ratio) {

			// If the fg part of the region is NOT a significant part of the mask...
			if(((double)rgn_info.m_fg_area / (double)mask_area) * 100.0f < m_fg_mask_ratio) {		
				// remove the fg part alone
				for(pixel_iter = rgn_info.m_pixel_list.begin(); pixel_iter != rgn_info.m_pixel_list.end(); pixel_iter++) {
					if(input_mask((*pixel_iter).x, (*pixel_iter).y)) {
						pixel_iter = rgn_info.m_pixel_list.erase(pixel_iter);
						pixel_iter--;
						rgn_info.m_total_area--;
					}
				}
				// Add it to the bg_rgns now
				RegionAttributes rgn_att;
				rgn_att.m_area = rgn_info.m_total_area;
				rgn_att.m_pixel_list = rgn_info.m_pixel_list;
				bg_rgns->push_back(rgn_att);
			}

			else {		// Divide the region into 2 and add one to the fg_rgns and another to bg_rgns
				RegionInfo fg_rgn_info;
				fg_rgn_info.m_total_area = 0;
				// Divide the region into 2
				for(pixel_iter = rgn_info.m_pixel_list.begin(); pixel_iter != rgn_info.m_pixel_list.end(); pixel_iter++) {
					if(input_mask((*pixel_iter).x, (*pixel_iter).y)) {
						// Add to the fg region info
						fg_rgn_info.m_pixel_list.push_back(*pixel_iter);
						fg_rgn_info.m_total_area++;
						// Remove from the bg region info
						pixel_iter = rgn_info.m_pixel_list.erase(pixel_iter);
						pixel_iter--;
						rgn_info.m_total_area--;
					}
				}
				// Add the 2 regions now
				RegionAttributes fg_rgn_att, bg_rgn_att;
				fg_rgn_att.m_area = fg_rgn_info.m_total_area;
				fg_rgn_att.m_pixel_list = fg_rgn_info.m_pixel_list;
				bg_rgn_att.m_area = rgn_info.m_total_area;
				bg_rgn_att.m_pixel_list = rgn_info.m_pixel_list;
				fg_rgns->push_back(fg_rgn_att);
				bg_rgns->push_back(bg_rgn_att);
			}
		}
		else {	// Add it to the fg_rgns
			RegionAttributes rgn_att;
			rgn_att.m_area = rgn_info.m_total_area;
			rgn_att.m_pixel_list = rgn_info.m_pixel_list;
			fg_rgns->push_back(rgn_att);
		}

	}	// for rgn_iter

}

/*
	This function is a helper function used by _GrowRegion to update the running statistics (sun n std dev)
	It updates the arguments - sum, sum_squares, size
*/
void RegionSegmentation::_UpdateRunningSum(Bgr val, vector<int>& sum, vector<int>& sum_squares, int& size)
{
	sum[0] += val.r;
	sum[1] += val.g;
	sum[2] += val.b;
	sum_squares[0] += (val.r * val.r);
	sum_squares[1] += (val.g * val.g);
	sum_squares[2] += (val.b * val.b);
	size++;
}


/*
	This function is another helper function used by _GrowRegion() to measure the similarity of a pixel with the current cluster
	It uses the running statistics - mean, std dev and size of the cluster, and the pixel value to make a decision. 
	If the pixel value is within 'thresh'*sigma of the cluster, then the function returns true. default value of 'thresh' is 3.
*/
bool RegionSegmentation::_IsSame(Bgr val1, Bgr seed_color, const vector<int>& sum, const vector<int>& sum_squares, int size)
{
	vector<float> mean, variance;
	int i;
	for(i=0; i<m_num_features; i++) {
		mean.push_back((float)sum[i]/(float)size);
		variance.push_back(((float)sum_squares[i]/(float)size) - (mean[i] * mean[i]));
	}

	float th1, th2, th3;
	if(size < 10) {
		th1 = 10.0f; th2 = 10.0f; th3 = 10.0f;
	}
	else {
		th1 = m_sigma_th * sqrt(variance[0]);
		th2 = m_sigma_th * sqrt(variance[1]);
		th3 = m_sigma_th * sqrt(variance[2]);
	}

	double seed_th = 1.4f;

	if((abs(val1.r-mean[0]) < th1) && (abs(val1.g-mean[1]) < th2) && (abs(val1.b-mean[2]) < th3) && (abs(seed_color.r-val1.r) < th1*seed_th) && (abs(seed_color.g-val1.g) < th2*seed_th) && (abs(seed_color.b-val1.b) < th3*seed_th))
	//if((abs(val1.r-mean[0]) < th1) && (abs(val1.g-mean[1]) < th2) && (abs(val1.b-mean[2]) < th3))
		return true;
	else
		return false;
}

/*
	It removes all small regions whose size is less than a threshold (m_area_th) and updates the label image and m_regions
	(region area / image size) * 100  <  m_area_th   -->  remove the region from m_regions
*/
void RegionSegmentation::_RemoveSmallRegions(int image_size, list<RegionInfo> *regions, ImgInt& labels)
{
	list<RegionInfo>::iterator list_iter;
	list<Point>::iterator pixel_iter;

	// Remove all regions whose size is too less and update the m_label image
	for(list_iter = regions->begin(); list_iter != regions->end(); list_iter++) {
		double perc = ((double)((*list_iter).m_total_area) / (double)image_size) * 100.0f;
		if(perc < m_area_th) {

			for(pixel_iter = (*list_iter).m_pixel_list.begin(); pixel_iter != (*list_iter).m_pixel_list.end(); pixel_iter++) {
				labels((*pixel_iter).x, (*pixel_iter).y) = -1;
			}

			list_iter = regions->erase(list_iter);
			list_iter--;
		}
	}
	//=================================================================
	/*
	ImgBgr pseudo_region;
	Utils::RandomColor(*m_labels, &pseudo_region);
	Figure fig;
	fig.Draw(pseudo_region);
	cout << m_regions->size() << endl;
	*/
	//=================================================================
}

void RegionSegmentation::_ClassifyEdgePixels(const ImgBgr& image, int width, int height)
{
	int x, y, xx, yy, i;
	vector<Point> pixel_vec;
	int wnd_size, curr_wnd_size;

	for(y=0; y<height; y++) {
		for(x=0; x<width; x++) {
			pixel_vec.clear();
			curr_wnd_size = 1;
			if((*m_labels)(x,y) == -1) {		// for each unclassified pixel in the label image

				while(pixel_vec.size() < 10) {		// collect atleast 3 segmented pixels in the neighborhood
					wnd_size = curr_wnd_size++;

					// Left column of the window
					xx = x - wnd_size;
					for(yy = y-wnd_size; yy <= y+wnd_size; yy++) {
						if(xx >= 0 && xx < width && yy >= 0 && yy < height && (*m_labels)(xx,yy) != -1)
							pixel_vec.push_back(Point(xx,yy));
					}

					// Top row of the window
					yy = y - wnd_size;
					for(xx = x-wnd_size+1; xx <= x+wnd_size; xx++) {
						if(xx >= 0 && xx < width && yy >= 0 && yy < height && (*m_labels)(xx,yy) != -1)
							pixel_vec.push_back(Point(xx,yy));
					}

					// Bottom row of the window
					yy = y + wnd_size;
					for(xx = x-wnd_size+1; xx <= x+wnd_size; xx++) {
						if(xx >= 0 && xx < width && yy >= 0 && yy < height && (*m_labels)(xx,yy) != -1)
							pixel_vec.push_back(Point(xx,yy));
					}

					// Right column of the window
					xx = x + wnd_size;
					for(yy = y-wnd_size+1; yy < y+wnd_size; yy++) {
						if(xx >= 0 && xx < width && yy >= 0 && yy < height && (*m_labels)(xx,yy) != -1)
							pixel_vec.push_back(Point(xx,yy));
					}
				}	// while()

				// Find the pixel with the closest similarity (euclidean distance) and assign its label
				double dist = 500.0f;
				Point chosen_pt;
				int size = pixel_vec.size();
				int r1 = image(x,y).r;        int g1 = image(x,y).g;        int b1 = image(x,y).b;
				for(i=0; i<size; i++) {
					Point pt = pixel_vec[i];
					int r2 = image(pt.x, pt.y).r; int g2 = image(pt.x, pt.y).g; int b2 = image(pt.x, pt.y).b;

					double curr_dist = sqrt((double)((r2-r1)*(r2-r1) + (g2-g1)*(g2-g1) + (b2-b1)*(b2-b1)));
					if(curr_dist < dist) {
						dist = curr_dist;
						chosen_pt = pt;
					}
				}
				(*m_labels)(x,y) = (*m_labels)(chosen_pt.x, chosen_pt.y);

			}	// if labels == -1
		}
	}
	//=================================================================
	/*
	ImgBgr pseudo_region;
	Utils::RandomColor(*m_labels, &pseudo_region);
	Figure fig;
	fig.Draw(pseudo_region);
	*/
	//=================================================================
}

void RegionSegmentation::SegmentSpecificRegions(const ImgBgr& image, const ImgBinary& mask, ImgInt& labels, vector<RegionAttributes> *regions)
{
	int pos, size;
	vector<Density> density_list;
	list<RegionInfo> region_info_list;

	_ComputeDensityMap(image, mask, &density_list);
	sort(density_list.begin(), density_list.end(), _comp);

	m_current_label_no = 1;
	pos = 0;
	size = density_list.size();
	while(pos < size) {
		if(labels(density_list[pos].m_pt.x, density_list[pos].m_pt.y) != 0) {	// if the pixel is already classified or edge, continue
			pos++;
			continue;
		}

		RegionInfo rgn_info;
		_GrowRegion(image, density_list[pos].m_pt, mask, rgn_info, labels);
		region_info_list.push_back(rgn_info);
		
		m_current_label_no++;
		pos++;
	}

	_RemoveSmallRegions(image.Width()*image.Height(), &region_info_list, labels);

	for(list<RegionInfo>::iterator list_iter = region_info_list.begin(); list_iter != region_info_list.end(); list_iter++) {
		RegionAttributes rgn_att;
		rgn_att.m_pixel_list = (*list_iter).m_pixel_list;
		rgn_att.m_area = (*list_iter).m_total_area;
		regions->push_back(rgn_att);
	}

}