
#include "BipartiteMatching.h"

void BipartiteMatching::DoBipartiteMatching(double **cost, int matrix_size, int *match_results, double& match_cost)
{
	_AllocateResource(matrix_size);

    match_cost = 0.0f;
    m_max_match = 0;                    //number of vertices in current matching
    memset(m_xy, -1, matrix_size*sizeof(int));    
    memset(m_yx, -1, matrix_size*sizeof(int));

    _InitLabels(cost, matrix_size);                    //step 0
    _Augment(cost, matrix_size);                        //steps 1-3

    for (int x = 0; x < matrix_size; x++) {      //forming answer there
        match_cost += cost[x][m_xy[x]];
		match_results[x] = m_xy[x];
	}

	_FreeResource();
}


void BipartiteMatching::_AllocateResource(int matrix_size)
{
	m_lx = new double[matrix_size];
	m_ly = new double[matrix_size];
	m_xy = new int[matrix_size];
	m_yx = new int[matrix_size];
	m_S = new bool[matrix_size];
	m_T = new bool[matrix_size];
	m_slack = new double[matrix_size];
	m_slackx = new int[matrix_size];
	m_prev = new int[matrix_size];
}

void BipartiteMatching::_FreeResource()
{
	delete [] m_lx;
	delete [] m_ly;
	delete [] m_xy;
	delete [] m_yx;
	delete [] m_S;
	delete [] m_T;
	delete [] m_slack;
	delete [] m_slackx;
	delete [] m_prev;
}

void BipartiteMatching::_InitLabels(double **cost, int matrix_size)
{
    memset(m_lx, 0, matrix_size*sizeof(double));
    memset(m_ly, 0, matrix_size*sizeof(double));
    for (int x = 0; x < matrix_size; x++)
        for (int y = 0; y < matrix_size; y++)
            m_lx[x] = max(m_lx[x], cost[x][y]);
}

void BipartiteMatching::_UpdateLabels(int matrix_size)
{
    int x, y;             //init delta as infinity
	double delta = INF;
    for (y = 0; y < matrix_size; y++)            //calculate delta using slack
        if (!m_T[y])
            delta = min(delta, m_slack[y]);
    for (x = 0; x < matrix_size; x++)            //update X labels
        if (m_S[x]) m_lx[x] -= delta;
    for (y = 0; y < matrix_size; y++)            //update Y labels
        if (m_T[y]) m_ly[y] += delta;
    for (y = 0; y < matrix_size; y++)            //update slack array
        if (!m_T[y])
            m_slack[y] -= delta;
}

//x - current vertex,prevx - vertex from X before x in the alternating path,
//so we add edges (prevx, xy[x]), (xy[x], x)
void BipartiteMatching::_AddToTree(int x, int prevx, double **cost, int matrix_size) 
{
    m_S[x] = true;                    //add x to S
    m_prev[x] = prevx;                //we need this when augmenting
    for (int y = 0; y < matrix_size; y++)    //update slacks, because we add new vertex to S
        if (m_lx[x] + m_ly[y] - cost[x][y] < m_slack[y])
        {
            m_slack[y] = m_lx[x] + m_ly[y] - cost[x][y];
            m_slackx[y] = x;
        }
}

void BipartiteMatching::_Augment(double **cost, int matrix_size)                         //main function of the algorithm
{
    if (m_max_match == matrix_size) return;        //check wether matching is already perfect
    int x, y, root;                    //just counters and root vertex
    int *q, wr = 0, rd = 0;          //q - queue for bfs, wr,rd - write and read
	q = new int[matrix_size];
                                       //pos in queue
    memset(m_S, false, matrix_size*sizeof(bool));       //init set S
    memset(m_T, false, matrix_size*sizeof(bool));       //init set T
    memset(m_prev, -1, matrix_size*sizeof(int));    //init set prev - for the alternating tree
    for (x = 0; x < matrix_size; x++)            //finding root of the tree
        if (m_xy[x] == -1)
        {
            q[wr++] = root = x;
            m_prev[x] = -2;
            m_S[x] = true;
            break;
        }

    for (y = 0; y < matrix_size; y++)            //initializing slack array
    {
        m_slack[y] = m_lx[root] + m_ly[y] - cost[root][y];
        m_slackx[y] = root;
    }

	//second part of augment() function
    while (true)                                                        //main cycle
    {
        while (rd < wr)                                                 //building tree with bfs cycle
        {
            x = q[rd++];                                                //current vertex from X part
            for (y = 0; y < matrix_size; y++)                                     //iterate through all edges in equality graph
                if (cost[x][y] == m_lx[x] + m_ly[y] &&  !m_T[y])
                {
                    if (m_yx[y] == -1) break;                             //an exposed vertex in Y found, so
                                                                        //augmenting path exists!
                    m_T[y] = true;                                        //else just add y to T,
                    q[wr++] = m_yx[y];                                    //add vertex yx[y], which is matched
                                                                        //with y, to the queue
                    _AddToTree(m_yx[y], x, cost, matrix_size);                              //add edges (x,y) and (y,yx[y]) to the tree
                }
            if (y < matrix_size) break;                                           //augmenting path found!
        }
        if (y < matrix_size) break;                                               //augmenting path found!

        _UpdateLabels(matrix_size);                                             //augmenting path not found, so improve labeling
        wr = rd = 0;                
        for (y = 0; y < matrix_size; y++)        
        //in this cycle we add edges that were added to the equality graph as a
        //result of improving the labeling, we add edge (slackx[y], y) to the tree if
        //and only if !T[y] &&  slack[y] == 0, also with this edge we add another one
        //(y, yx[y]) or augment the matching, if y was exposed
            if (!m_T[y] &&  m_slack[y] == 0)
            {
                if (m_yx[y] == -1)                                        //exposed vertex in Y found - augmenting path exists!
                {
                    x = m_slackx[y];
                    break;
                }
                else
                {
                    m_T[y] = true;                                        //else just add y to T,
                    if (!m_S[m_yx[y]])    
                    {
                        q[wr++] = m_yx[y];                                //add vertex yx[y], which is matched with
                                                                        //y, to the queue
                        _AddToTree(m_yx[y], m_slackx[y], cost, matrix_size);                  //and add edges (x,y) and (y,
                                                                        //yx[y]) to the tree
                    }
                }
            }
        if (y < matrix_size) break;                                               //augmenting path found!
    }

    if (y < matrix_size)                                                          //we found augmenting path!
    {
        m_max_match++;                                                    //increment matching
        //in this cycle we inverse edges along augmenting path
        for (int cx = x, cy = y, ty; cx != -2; cx = m_prev[cx], cy = ty)
        {
            ty = m_xy[cx];
            m_yx[cy] = cx;
            m_xy[cx] = cy;
        }
        _Augment(cost, matrix_size);                                                      //recall function, go to step 1 of the algorithm
    }
	delete [] q;
}//end of augment() function
